"""A module that checks a patent document.

チェック処理のコア部分
    - 実際に行うチェック処理(ルール)のインスタンス生成と実行
    - 実行結果のまとめ
    - 出力
"""

from __future__ import print_function
import os

from formatters import print_formatter
from config import config_loader
from rules import rule_executor
from astree.parsing import Parser
from typing import Dict, List, Any


def check_text(text_file_path: str, config_file_path: str) -> List[Dict[str, Any]]:
    """
    Check docstring of single Python module.

    Parameters
    ----------
    text_file_path : str
        Path of target module.

    Returns
    -------
    info_list : list of dicts
        A list containing information on check results.

    Raises
    ------
    IOError
        If the target text file can not be found.
    """
    p = Parser()
    astree = p.parse(text_file_path)

    info_list: List[Dict[str, Any]] = []
    if config_file_path is not None:
        loader = config_loader.Config_loader(config_file_path)
        if loader.exist_rules():
            rule_dict = loader.get_enabled_rules_dict()
            for rule_name in rule_dict:
                rule_config_dict = rule_dict[rule_name]
                info = rule_executor.rule_executor(
                    rule_name, text_file_path, astree, rule_config_dict
                )
                info_list.extend(info)
    else:
        default_rule_set = [
            "Check_sentence_length",
            "Check_multiple_predicate",
            "Check_multiple_subject",
            "Check_dependency_pattern",
            "Check_difficult_word",
            "Check_ng_word",
        ]
        for rule_name in default_rule_set:
            info = rule_executor.rule_executor(rule_name, text_file_path, astree, {})
            info_list.extend(info)

    formatter = print_formatter.Print_formatter()
    formatter.print_info_list(info_list=info_list)
    return info_list


def _check_text_exists(text_file_path):
    """
    Check that the target text file exists.

    Parameters
    ----------
    text_file_path : str
        Path of target text file.

    Raises
    ------
    IOError
        If the target module can not be found.
    """
    if os.path.exists(text_file_path):
        return
    err_msg = "The target text file could not be found."
    err_msg += "\ntext file: %s" % text_file_path
    raise IOError(err_msg)
